@@ -11,6 +11,7 @@ use crate::{
 };
 use std::{
 	borrow::Borrow,
+	cmp::Ordering,
 	convert::TryFrom,
 	fmt,
 	ops::Deref,
@@ -102,7 +103,7 @@ impl Borrow<str> for FmtUtc2k {
 
 impl Default for FmtUtc2k {
 	#[inline]
-	fn default() -> Self { Self(Self::MIN) }
+	fn default() -> Self { Self::min() }
 }
 
 impl Deref for FmtUtc2k {
@@ -131,6 +132,16 @@ impl From<Utc2k> for FmtUtc2k {
 	}
 }
 
+impl Ord for FmtUtc2k {
+	#[inline]
+	fn cmp(&self, other: &Self) -> Ordering { self.0.cmp(&other.0) }
+}
+
+impl PartialOrd for FmtUtc2k {
+	#[inline]
+	fn partial_cmp(&self, other: &Self) -> Option<Ordering> { Some(self.cmp(other)) }
+}
+
 impl TryFrom<&str> for FmtUtc2k {
 	type Error = Utc2kError;
 
@@ -158,6 +169,20 @@ impl FmtUtc2k {
 	/// This returns an instance using the current unixtime as the seed.
 	pub fn now() -> Self { Self::from(Utc2k::now()) }
 
+	#[inline]
+	#[must_use]
+	/// # Maximum Value.
+	///
+	/// This is equivalent to `2099-12-31 23:59:59`.
+	pub const fn max() -> Self { Self(Self::MAX) }
+
+	#[inline]
+	#[must_use]
+	/// # Minimum Value.
+	///
+	/// This is equivalent to `2000-01-01 00:00:00`.
+	pub const fn min() -> Self { Self(Self::MIN) }
+
 	#[allow(clippy::cast_possible_truncation)] // It fits.
 	/// # Set Date/Time.
 	///
@@ -355,16 +380,7 @@ pub struct Utc2k {
 
 impl Default for Utc2k {
 	#[inline]
-	fn default() -> Self {
-		Self {
-			y: 0,
-			m: 1,
-			d: 1,
-			hh: 0,
-			mm: 0,
-			ss: 0,
-		}
-	}
+	fn default() -> Self { Self::min() }
 }
 
 impl fmt::Display for Utc2k {
@@ -376,7 +392,6 @@ impl fmt::Display for Utc2k {
 
 impl From<u32> for Utc2k {
 	#[allow(clippy::integer_division)]
-	#[allow(clippy::cast_possible_truncation)] // It fits.
 	/// # From Timestamp.
 	///
 	/// Note, this will saturate to [`Utc2k::MIN_UNIXTIME`] and
@@ -391,34 +406,44 @@ impl From<u32> for Utc2k {
 	/// assert_eq!(Utc2k::from(u32::MAX).to_string(), "2099-12-31 23:59:59");
 	/// ```
 	fn from(src: u32) -> Self {
-		if src < Self::MIN_UNIXTIME { return Self::default(); }
-		else if src > Self::MAX_UNIXTIME {
-			return Self { y: 99, m: 12, d: 31, hh: 23, mm: 59, ss: 59 };
-		}
+		if src < Self::MIN_UNIXTIME { return Self::min(); }
+		else if src > Self::MAX_UNIXTIME { return Self::max(); }
 
-		let (div, rem) = div_mod(src, 86_400);
-		let d = div + 719_468;
-		let y = (d - (d + 2 + 3 * d / 146_097) / 1_461 + (d - d / 146_097) / 36_524 - (d + 1) / 146_097) / 365;
-		let y_rem = d - (365 * y + y / 4 - y / 100 + y / 400);
-		let m = (y_rem - (y_rem + 20) / 50) / 30;
-		let day = y_rem - (30 * m + 3 * (m + 4) / 5 - 2) + 1;
-		let m_cont = 12 * y + m + 2;
-		let year = m_cont / 12;
-		let month = m_cont % 12 + 1;
-		let (hour, h_rem) = div_mod(rem, 3_600);
-		let (minute, second) = div_mod(h_rem, 60);
+		// Tease out the date parts with a lot of terrible math.
+		let (y, m, d) = parse_date_seconds(src / DAY_IN_SECONDS);
+		let (hh, mm, ss) = parse_time_seconds(src % DAY_IN_SECONDS);
 
-		Self {
-			y: (year - 2000) as u8,
-			m: month as u8,
-			d: day as u8,
-			hh: hour as u8,
-			mm: minute as u8,
-			ss: second as u8,
+		Self { y, m, d, hh, mm, ss }
+	}
+}
+
+impl Ord for Utc2k {
+	fn cmp(&self, other: &Self) -> Ordering {
+		// Work our way down until there's a difference!
+		match self.y.cmp(&other.y) {
+			Ordering::Equal => match self.m.cmp(&other.m) {
+				Ordering::Equal => match self.d.cmp(&other.d) {
+					Ordering::Equal => match self.hh.cmp(&other.hh) {
+						Ordering::Equal => match self.mm.cmp(&other.mm) {
+							Ordering::Equal => self.ss.cmp(&other.ss),
+							x => x,
+						},
+						x => x,
+					},
+					x => x,
+				},
+				x => x,
+			},
+			x => x,
 		}
 	}
 }
 
+impl PartialOrd for Utc2k {
+	#[inline]
+	fn partial_cmp(&self, other: &Self) -> Option<Ordering> { Some(self.cmp(other)) }
+}
+
 try_from_unixtime!(i32, u64, i64, usize, isize);
 
 impl TryFrom<&str> for Utc2k {
@@ -451,6 +476,20 @@ impl Utc2k {
 
 /// ## Instantiation.
 impl Utc2k {
+	#[inline]
+	#[must_use]
+	/// # Maximum Value.
+	///
+	/// This is equivalent to `2099-12-31 23:59:59`.
+	pub const fn max() -> Self { Self { y: 99, m: 12, d: 31, hh: 23, mm: 59, ss: 59 } }
+
+	#[inline]
+	#[must_use]
+	/// # Minimum Value.
+	///
+	/// This is equivalent to `2000-01-01 00:00:00`.
+	pub const fn min() -> Self { Self { y: 0, m: 1, d: 1, hh: 0, mm: 0, ss: 0 } }
+
 	#[allow(clippy::cast_possible_truncation)] // It fits.
 	#[must_use]
 	/// # New (From Parts).
@@ -560,20 +599,18 @@ impl Utc2k {
 		// Convert the year to its full four-digit form.
 		let y: u16 = self.year();
 
-		// Seconds from this month.
-		let mut time = u32::from(self.ss) +
+		// Start with all the seconds prior to the year.
+		let (mut time, leap) = year_size(y);
+
+		// Add up all the month seconds.
+		time += u32::from(self.ss) +
 			MINUTE_IN_SECONDS * u32::from(self.mm) +
 			HOUR_IN_SECONDS * u32::from(self.hh) +
-			DAY_IN_SECONDS * (u32::from(self.d) - 1);
-
-		// Seconds from all the years before this year.
-		time += year_size(y);
-
-		// Seconds from the months prior to this month.
-		time += month_size(self.m);
+			DAY_IN_SECONDS * (u32::from(self.d) - 1) +
+			month_size(self.m);
 
 		// Add an extra leap day?
-		if self.m > 2 && leap_year(y) {
+		if self.m > 2 && leap {
 			time += DAY_IN_SECONDS;
 		}
 
@@ -695,25 +732,10 @@ const fn carry_over_date_parts(mut y: u16, mut m: u8, mut d: u8) -> (u16, u8, u8
 	(y, m, d)
 }
 
-#[allow(clippy::integer_division)]
-#[inline]
-/// # Combined Division and Mod.
-const fn div_mod(left: u32, right: u32) -> (u32, u32) { (left / right, left % right) }
-
 #[inline]
 #[must_use]
 /// # Is Leap Year?
-///
-/// This is pre-computed for years in our centuries, otherwise it falls back to
-/// a simple calculation.
-const fn leap_year(y: u16) -> bool {
-	if y >= 2000 {
-		matches!(y, 2000 | 2004 | 2008 | 2012 | 2016 | 2020 | 2024 | 2028 | 2032 | 2036 | 2040 | 2044 | 2048 | 2052 | 2056 | 2060 | 2064 | 2068 | 2072 | 2076 | 2080 | 2084 | 2088 | 2092 | 2096)
-	}
-	else {
-		(y % 4 == 0 && y % 100 != 0) || y % 400 == 0
-	}
-}
+const fn leap_year(y: u16) -> bool { (y % 4 == 0 && y % 100 != 0) || y % 400 == 0 }
 
 #[must_use]
 /// # Days in Month.
@@ -755,6 +777,48 @@ const fn month_size(m: u8) -> u32 {
 	}
 }
 
+#[allow(clippy::cast_possible_truncation)] // It fits.
+#[allow(clippy::integer_division)]
+/// # Parse Date From Seconds.
+///
+/// This is pre-divided into an even number of days, so will produce the parts.
+const fn parse_date_seconds(mut src: u32) -> (u8, u8, u8) {
+	src += 719_468;
+	let y = (src - (src + 2 + 3 * src / 146_097) / 1_461 + (src - src / 146_097) / 36_524 - (src + 1) / 146_097) / 365;
+	src -= 365 * y + y / 4 - y / 100 + y / 400;
+	let m = (src - (src + 20) / 50) / 30;
+	let m_cont = 12 * y + m + 2;
+	(
+		(m_cont / 12 - 2000) as u8,
+		(m_cont % 12 + 1) as u8,
+		(src - (30 * m + 3 * (m + 4) / 5 - 2) + 1) as u8
+	)
+}
+
+#[allow(clippy::cast_possible_truncation)] // It fits.
+/// # Parse Time From Seconds.
+///
+/// Days have been eliminated already, so this will produce the parts.
+const fn parse_time_seconds(mut src: u32) -> (u8, u8, u8) {
+	let hh =
+		if src >= HOUR_IN_SECONDS {
+			let hh = ((src * 0x91A3) >> 27) as u8;
+			src -= hh as u32 * HOUR_IN_SECONDS;
+			hh
+		}
+		else { 0 };
+
+	let mm =
+		if src >= MINUTE_IN_SECONDS {
+			let mm = ((src * 0x889) >> 17) as u8;
+			src -= mm as u32 * MINUTE_IN_SECONDS;
+			mm
+		}
+		else { 0 };
+
+	(hh, mm, src as u8)
+}
+
 #[allow(clippy::too_many_lines)] // We have a century to cover!
 #[must_use]
 /// # Year Size.
@@ -762,109 +826,111 @@ const fn month_size(m: u8) -> u32 {
 /// This returns the number of seconds from 1970 up to a given year. Since
 /// we're focusing on a single century, this can be presented as a pre-computed
 /// table for fast lookups.
-const fn year_size(y: u16) -> u32 {
+///
+/// The second value indicates whether or not this is a leap year.
+const fn year_size(y: u16) -> (u32, bool) {
 	match y {
-		2000 => 946_684_800,
-		2001 => 978_307_200,
-		2002 => 1_009_843_200,
-		2003 => 1_041_379_200,
-		2004 => 1_072_915_200,
-		2005 => 1_104_537_600,
-		2006 => 1_136_073_600,
-		2007 => 1_167_609_600,
-		2008 => 1_199_145_600,
-		2009 => 1_230_768_000,
-		2010 => 1_262_304_000,
-		2011 => 1_293_840_000,
-		2012 => 1_325_376_000,
-		2013 => 1_356_998_400,
-		2014 => 1_388_534_400,
-		2015 => 1_420_070_400,
-		2016 => 1_451_606_400,
-		2017 => 1_483_228_800,
-		2018 => 1_514_764_800,
-		2019 => 1_546_300_800,
-		2020 => 1_577_836_800,
-		2021 => 1_609_459_200,
-		2022 => 1_640_995_200,
-		2023 => 1_672_531_200,
-		2024 => 1_704_067_200,
-		2025 => 1_735_689_600,
-		2026 => 1_767_225_600,
-		2027 => 1_798_761_600,
-		2028 => 1_830_297_600,
-		2029 => 1_861_920_000,
-		2030 => 1_893_456_000,
-		2031 => 1_924_992_000,
-		2032 => 1_956_528_000,
-		2033 => 1_988_150_400,
-		2034 => 2_019_686_400,
-		2035 => 2_051_222_400,
-		2036 => 2_082_758_400,
-		2037 => 2_114_380_800,
-		2038 => 2_145_916_800,
-		2039 => 2_177_452_800,
-		2040 => 2_208_988_800,
-		2041 => 2_240_611_200,
-		2042 => 2_272_147_200,
-		2043 => 2_303_683_200,
-		2044 => 2_335_219_200,
-		2045 => 2_366_841_600,
-		2046 => 2_398_377_600,
-		2047 => 2_429_913_600,
-		2048 => 2_461_449_600,
-		2049 => 2_493_072_000,
-		2050 => 2_524_608_000,
-		2051 => 2_556_144_000,
-		2052 => 2_587_680_000,
-		2053 => 2_619_302_400,
-		2054 => 2_650_838_400,
-		2055 => 2_682_374_400,
-		2056 => 2_713_910_400,
-		2057 => 2_745_532_800,
-		2058 => 2_777_068_800,
-		2059 => 2_808_604_800,
-		2060 => 2_840_140_800,
-		2061 => 2_871_763_200,
-		2062 => 2_903_299_200,
-		2063 => 2_934_835_200,
-		2064 => 2_966_371_200,
-		2065 => 2_997_993_600,
-		2066 => 3_029_529_600,
-		2067 => 3_061_065_600,
-		2068 => 3_092_601_600,
-		2069 => 3_124_224_000,
-		2070 => 3_155_760_000,
-		2071 => 3_187_296_000,
-		2072 => 3_218_832_000,
-		2073 => 3_250_454_400,
-		2074 => 3_281_990_400,
-		2075 => 3_313_526_400,
-		2076 => 3_345_062_400,
-		2077 => 3_376_684_800,
-		2078 => 3_408_220_800,
-		2079 => 3_439_756_800,
-		2080 => 3_471_292_800,
-		2081 => 3_502_915_200,
-		2082 => 3_534_451_200,
-		2083 => 3_565_987_200,
-		2084 => 3_597_523_200,
-		2085 => 3_629_145_600,
-		2086 => 3_660_681_600,
-		2087 => 3_692_217_600,
-		2088 => 3_723_753_600,
-		2089 => 3_755_376_000,
-		2090 => 3_786_912_000,
-		2091 => 3_818_448_000,
-		2092 => 3_849_984_000,
-		2093 => 3_881_606_400,
-		2094 => 3_913_142_400,
-		2095 => 3_944_678_400,
-		2096 => 3_976_214_400,
-		2097 => 4_007_836_800,
-		2098 => 4_039_372_800,
-		2099 => 4_070_908_800,
-		_ => 0,
+		2000 => (946_684_800, true),
+		2001 => (978_307_200, false),
+		2002 => (1_009_843_200, false),
+		2003 => (1_041_379_200, false),
+		2004 => (1_072_915_200, true),
+		2005 => (1_104_537_600, false),
+		2006 => (1_136_073_600, false),
+		2007 => (1_167_609_600, false),
+		2008 => (1_199_145_600, true),
+		2009 => (1_230_768_000, false),
+		2010 => (1_262_304_000, false),
+		2011 => (1_293_840_000, false),
+		2012 => (1_325_376_000, true),
+		2013 => (1_356_998_400, false),
+		2014 => (1_388_534_400, false),
+		2015 => (1_420_070_400, false),
+		2016 => (1_451_606_400, true),
+		2017 => (1_483_228_800, false),
+		2018 => (1_514_764_800, false),
+		2019 => (1_546_300_800, false),
+		2020 => (1_577_836_800, true),
+		2021 => (1_609_459_200, false),
+		2022 => (1_640_995_200, false),
+		2023 => (1_672_531_200, false),
+		2024 => (1_704_067_200, true),
+		2025 => (1_735_689_600, false),
+		2026 => (1_767_225_600, false),
+		2027 => (1_798_761_600, false),
+		2028 => (1_830_297_600, true),
+		2029 => (1_861_920_000, false),
+		2030 => (1_893_456_000, false),
+		2031 => (1_924_992_000, false),
+		2032 => (1_956_528_000, true),
+		2033 => (1_988_150_400, false),
+		2034 => (2_019_686_400, false),
+		2035 => (2_051_222_400, false),
+		2036 => (2_082_758_400, true),
+		2037 => (2_114_380_800, false),
+		2038 => (2_145_916_800, false),
+		2039 => (2_177_452_800, false),
+		2040 => (2_208_988_800, true),
+		2041 => (2_240_611_200, false),
+		2042 => (2_272_147_200, false),
+		2043 => (2_303_683_200, false),
+		2044 => (2_335_219_200, true),
+		2045 => (2_366_841_600, false),
+		2046 => (2_398_377_600, false),
+		2047 => (2_429_913_600, false),
+		2048 => (2_461_449_600, true),
+		2049 => (2_493_072_000, false),
+		2050 => (2_524_608_000, false),
+		2051 => (2_556_144_000, false),
+		2052 => (2_587_680_000, true),
+		2053 => (2_619_302_400, false),
+		2054 => (2_650_838_400, false),
+		2055 => (2_682_374_400, false),
+		2056 => (2_713_910_400, true),
+		2057 => (2_745_532_800, false),
+		2058 => (2_777_068_800, false),
+		2059 => (2_808_604_800, false),
+		2060 => (2_840_140_800, true),
+		2061 => (2_871_763_200, false),
+		2062 => (2_903_299_200, false),
+		2063 => (2_934_835_200, false),
+		2064 => (2_966_371_200, true),
+		2065 => (2_997_993_600, false),
+		2066 => (3_029_529_600, false),
+		2067 => (3_061_065_600, false),
+		2068 => (3_092_601_600, true),
+		2069 => (3_124_224_000, false),
+		2070 => (3_155_760_000, false),
+		2071 => (3_187_296_000, false),
+		2072 => (3_218_832_000, true),
+		2073 => (3_250_454_400, false),
+		2074 => (3_281_990_400, false),
+		2075 => (3_313_526_400, false),
+		2076 => (3_345_062_400, true),
+		2077 => (3_376_684_800, false),
+		2078 => (3_408_220_800, false),
+		2079 => (3_439_756_800, false),
+		2080 => (3_471_292_800, true),
+		2081 => (3_502_915_200, false),
+		2082 => (3_534_451_200, false),
+		2083 => (3_565_987_200, false),
+		2084 => (3_597_523_200, true),
+		2085 => (3_629_145_600, false),
+		2086 => (3_660_681_600, false),
+		2087 => (3_692_217_600, false),
+		2088 => (3_723_753_600, true),
+		2089 => (3_755_376_000, false),
+		2090 => (3_786_912_000, false),
+		2091 => (3_818_448_000, false),
+		2092 => (3_849_984_000, true),
+		2093 => (3_881_606_400, false),
+		2094 => (3_913_142_400, false),
+		2095 => (3_944_678_400, false),
+		2096 => (3_976_214_400, true),
+		2097 => (4_007_836_800, false),
+		2098 => (4_039_372_800, false),
+		2099 => (4_070_908_800, false),
+		_ => (0, leap_year(y)),
 	}
 }
 
@@ -966,4 +1032,41 @@ mod tests {
 			(2099, 12, 31, 23, 59, 59)
 		);
 	}
+
+	#[test]
+	/// # Test Ordering.
+	fn ordering() {
+		let expected = vec![
+			Utc2k::try_from("2000-01-01 00:00:00").unwrap(),
+			Utc2k::try_from("2010-05-31 01:02:03").unwrap(),
+			Utc2k::try_from("2010-05-31 02:02:03").unwrap(),
+			Utc2k::try_from("2020-10-10 10:10:10").unwrap(),
+			Utc2k::try_from("2020-10-10 10:11:10").unwrap(),
+			Utc2k::try_from("2020-10-10 10:11:11").unwrap(),
+		];
+
+		let mut shuffled = vec![
+			Utc2k::try_from("2010-05-31 01:02:03").unwrap(),
+			Utc2k::try_from("2020-10-10 10:11:11").unwrap(),
+			Utc2k::try_from("2010-05-31 02:02:03").unwrap(),
+			Utc2k::try_from("2000-01-01 00:00:00").unwrap(),
+			Utc2k::try_from("2020-10-10 10:11:10").unwrap(),
+			Utc2k::try_from("2020-10-10 10:10:10").unwrap(),
+		];
+
+		let f_expected: Vec<FmtUtc2k> = expected.iter().copied().map(FmtUtc2k::from).collect();
+		let mut f_shuffled: Vec<FmtUtc2k> = shuffled.iter().copied().map(FmtUtc2k::from).collect();
+
+		// Both sets should be not equal to start.
+		assert_ne!(expected, shuffled);
+		assert_ne!(f_expected, f_shuffled);
+
+		// Sort 'em.
+		shuffled.sort();
+		f_shuffled.sort();
+
+		// Now they should match.
+		assert_eq!(expected, shuffled);
+		assert_eq!(f_expected, f_shuffled);
+	}
 }
